@@ -81,7 +81,7 @@ def build_cluster_endpoint(
         return custom_endpoint.endpoint
 
     if config.ES_ENDPOINT_STRATEGY == "off":
-        return "%s:%s" % (config.LOCALSTACK_HOSTNAME, config.PORT_ELASTICSEARCH)
+        return f"{config.LOCALSTACK_HOSTNAME}:{config.service_port('elasticsearch')}"
     if config.ES_ENDPOINT_STRATEGY == "path":
         return "%s:%s/es/%s/%s" % (
             config.LOCALSTACK_HOSTNAME,
@@ -116,7 +116,7 @@ class ClusterManager:
     clusters: Dict[str, Server]
 
     def __init__(self) -> None:
-        self.clusters = dict()
+        self.clusters = {}
 
     def create(self, arn: str, create_domain_request: Dict) -> Server:
         version = versions.get_install_version(
@@ -195,7 +195,7 @@ class SingletonClusterManager(ClusterManager):
             #  shut down) there's a chance that there will be a bind exception when trying to start the proxy again
             #  (which is currently always bound to PORT_ELASTICSEARCH)
             self.cluster = ProxiedElasticsearchCluster(
-                port=config.PORT_ELASTICSEARCH,
+                port=config.service_port("elasticsearch"),
                 host=constants.LOCALHOST,
                 version=version,
                 directories=resolve_directories(version, arn),
@@ -246,7 +246,7 @@ class MultiplexingClusterManager(ClusterManager):
     def __init__(self) -> None:
         super().__init__()
         self.cluster = None
-        self.endpoints = dict()
+        self.endpoints = {}
         self.mutex = threading.RLock()
 
     def _create_cluster(self, arn, url, version, create_domain_request) -> Server:
