@@ -79,9 +79,11 @@ def es():
 
 @aws_provider()
 def firehose():
-    from localstack.services.firehose import firehose_starter
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services.firehose.provider import FirehoseProvider
 
-    return Service("firehose", start=firehose_starter.start_firehose)
+    provider = FirehoseProvider()
+    return Service("firehose", listener=AwsApiListener("firehose", provider))
 
 
 @aws_provider()
@@ -138,6 +140,15 @@ def logs():
     )
 
 
+@aws_provider()
+def opensearch():
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services.opensearch.provider import OpensearchProvider
+
+    provider = OpensearchProvider()
+    return Service("opensearch", listener=AwsApiListener("opensearch", provider))
+
+
 @aws_provider()
 def redshift():
     from localstack.services.redshift import redshift_starter
@@ -208,6 +219,16 @@ def sqs():
     )
 
 
+@aws_provider(api="sqs", name="asf")
+def sqs_asf():
+    from localstack.aws.proxy import AwsApiListener
+    from localstack.services.sqs.provider import SqsProvider
+
+    provider = SqsProvider()
+
+    return Service("sqs", listener=AwsApiListener("sqs", provider), lifecycle_hook=provider)
+
+
 @aws_provider()
 def ssm():
     from localstack.services.ssm import ssm_listener, ssm_starter
@@ -238,13 +259,11 @@ def stepfunctions():
 
 @aws_provider()
 def swf():
-    from localstack.services.swf import swf_listener, swf_starter
+    from localstack.services.swf import swf_starter
 
     return Service(
         "swf",
-        listener=swf_listener.UPDATE_SWF,
         start=swf_starter.start_swf,
-        check=swf_starter.check_swf,
     )
 
 
