@@ -7,9 +7,8 @@ import json
 import logging
 import random
 import re
-import urllib.parse
 import uuid
-from urllib.parse import parse_qs
+from urllib.parse import parse_qs, parse_qsl, quote, unquote, urlencode, urlparse, urlunparse
 
 import botocore.config
 import dateutil.parser
@@ -20,7 +19,6 @@ from moto.s3.exceptions import InvalidFilterRuleName
 from moto.s3.models import s3_backend
 from pytz import timezone
 from requests.models import Request, Response
-from six.moves.urllib import parse as urlparse
 
 from localstack import config, constants
 from localstack.services.s3 import multipart_content
@@ -155,7 +153,7 @@ def filter_rules_match(filters, object_path):
             if not object_path.endswith(rule["Value"]):
                 return False
         else:
-            LOGGER.warning('Unknown filter name: "%s"' % rule["Name"])
+            LOGGER.warning('Unknown filter name: "%s"', rule["Name"])
     return True
 
 
@@ -202,7 +200,7 @@ def get_event_message(
                         "arn": "arn:aws:s3:::%s" % bucket_name,
                     },
                     "object": {
-                        "key": urllib.parse.quote(file_name),
+                        "key": quote(file_name),
                         "size": file_size,
                         "eTag": etag,
                         "versionId": version_id,
@@ -261,13 +259,12 @@ def send_notification_for_subscriber(
     ):
         return
 
-    key = urlparse.unquote(object_path.replace("//", "/"))[1:]
+    key = unquote(object_path.replace("//", "/"))[1:]
 
     s3_client = aws_stack.connect_to_service("s3")
     object_data = {}
     try:
         object_data = s3_client.head_object(Bucket=bucket_name, Key=key)
-
     except botocore.exceptions.ClientError:
         pass
 
@@ -294,8 +291,10 @@ def send_notification_for_subscriber(
             )
         except Exception as e:
             LOGGER.warning(
-                'Unable to send notification for S3 bucket "%s" to SQS queue "%s": %s'
-                % (bucket_name, notif["Queue"], e)
+                'Unable to send notification for S3 bucket "%s" to SQS queue "%s": %s',
+                bucket_name,
+                notif["Queue"],
+                e,
             )
     if notif.get("Topic"):
         region = aws_stack.extract_region_from_arn(notif["Topic"])
@@ -308,8 +307,10 @@ def send_notification_for_subscriber(
             )
         except Exception as e:
             LOGGER.warning(
-                'Unable to send notification for S3 bucket "%s" to SNS topic "%s": %s'
-                % (bucket_name, notif["Topic"], e)
+                'Unable to send notification for S3 bucket "%s" to SNS topic "%s": %s',
+                bucket_name,
+                notif["Topic"],
+                e,
             )
     # CloudFunction and LambdaFunction are semantically identical
     lambda_function_config = notif.get("CloudFunction") or notif.get("LambdaFunction")
@@ -328,13 +329,14 @@ def send_notification_for_subscriber(
             )
         except Exception:
             LOGGER.warning(
-                'Unable to send notification for S3 bucket "%s" to Lambda function "%s".'
-                % (bucket_name, lambda_function_config)
+                'Unable to send notification for S3 bucket "%s" to Lambda function "%s".',
+                bucket_name,
+                lambda_function_config,
             )
 
     if not filter(lambda x: notif.get(x), NOTIFICATION_DESTINATION_TYPES):
         LOGGER.warning(
-            "Neither of %s defined for S3 notification." % "/".join(NOTIFICATION_DESTINATION_TYPES)
+            "Neither of %s defined for S3 notification.", "/".join(NOTIFICATION_DESTINATION_TYPES)
         )
 
 
@@ -592,18 +594,18 @@ def append_last_modified_headers(response, content=None):
                 )
                 response.headers["Last-Modified"] = last_modified_time_format
     except TypeError as err:
-        LOGGER.debug("No parsable content: %s" % err)
+        LOGGER.debug("No parsable content: %s", err)
     except ValueError as err:
-        LOGGER.error("Failed to parse LastModified: %s" % err)
+        LOGGER.error("Failed to parse LastModified: %s", err)
     except Exception as err:
-        LOGGER.error("Caught generic exception (parsing LastModified): %s" % err)
+        LOGGER.error("Caught generic exception (parsing LastModified): %s", err)
     # if cannot parse any LastModified, just continue
 
     try:
         if response.headers.get("Last-Modified", "") == "":
             response.headers["Last-Modified"] = datetime.datetime.now().strftime(time_format)
     except Exception as err:
-        LOGGER.error("Caught generic exception (setting LastModified header): %s" % err)
+        LOGGER.error("Caught generic exception (setting LastModified header): %s", err)
 
 
 def append_list_objects_marker(method, path, data, response):
@@ -611,8 +613,8 @@ def append_list_objects_marker(method, path, data, response):
         marker = ""
         content = to_str(response.content)
         if "<ListBucketResult" in content and "<Marker>" not in content:
-            parsed = urlparse.urlparse(path)
-            query_map = urlparse.parse_qs(parsed.query)
+            parsed = urlparse(path)
+            query_map = parse_qs(parsed.query)
             if query_map.get("marker") and query_map.get("marker")[0]:
                 marker = query_map.get("marker")[0]
             insert = "<Marker>%s</Marker>" % marker
@@ -651,7 +653,7 @@ def fix_range_content_type(bucket_name, path, headers, response):
         return
 
     s3_client = aws_stack.connect_to_service("s3")
-    path = urlparse.urlparse(urlparse.unquote(path)).path
+    path = urlparse(unquote(path)).path
     key_name = extract_key_name(headers, path)
     result = s3_client.head_object(Bucket=bucket_name, Key=key_name)
     content_type = result["ContentType"]
@@ -687,7 +689,9 @@ def fix_delete_objects_response(bucket_name, method, parsed_path, data, headers,
     response._content = xmltodict.unparse({"DeleteResult": result})
 
 
-def fix_metadata_key_underscores(request_headers={}, response=None):
+def fix_metadata_key_underscores(request_headers=None, response=None):
+    if request_headers is None:
+        request_headers = {}
     # fix for https://github.com/localstack/localstack/issues/1790
     underscore_replacement = "---"
     meta_header_prefix = "x-amz-meta-"
@@ -739,7 +743,7 @@ def convert_to_chunked_encoding(method, path, response):
     response.headers.pop("Content-Length", None)
 
 
-def unquote(s):
+def strip_surrounding_quotes(s):
     if (s[0], s[-1]) in (('"', '"'), ("'", "'")):
         return s[1:-1]
     return s
@@ -749,7 +753,7 @@ def ret304_on_etag(data, headers, response):
     etag = response.headers.get("ETag")
     if etag:
         match = headers.get("If-None-Match")
-        if match and unquote(match) == unquote(etag):
+        if match and strip_surrounding_quotes(match) == strip_surrounding_quotes(etag):
             response.status_code = 304
             response._content = ""
 
@@ -991,17 +995,17 @@ def token_expired_error(resource, requestId=None, status_code=400):
 
 def expand_redirect_url(starting_url, key, bucket):
     """Add key and bucket parameters to starting URL query string."""
-    parsed = urlparse.urlparse(starting_url)
-    query = collections.OrderedDict(urlparse.parse_qsl(parsed.query))
+    parsed = urlparse(starting_url)
+    query = collections.OrderedDict(parse_qsl(parsed.query))
     query.update([("key", key), ("bucket", bucket)])
 
-    redirect_url = urlparse.urlunparse(
+    redirect_url = urlunparse(
         (
             parsed.scheme,
             parsed.netloc,
             parsed.path,
             parsed.params,
-            urlparse.urlencode(query),
+            urlencode(query),
             None,
         )
     )
@@ -1163,7 +1167,7 @@ class ProxyListenerS3(PersistingProxyListener):
                 """.format(
             protocol=get_service_protocol(),
             host=config.HOSTNAME_EXTERNAL,
-            encoded_key=urlparse.quote(key, safe=""),
+            encoded_key=quote(key, safe=""),
             key=key,
             bucket=bucket_name,
             etag="d41d8cd98f00b204e9800998ecf8427f",
@@ -1175,7 +1179,7 @@ class ProxyListenerS3(PersistingProxyListener):
 
         host = config.HOSTNAME_EXTERNAL
         if ":" not in host:
-            host = "%s:%s" % (host, config.PORT_S3)
+            host = f"{host}:{config.service_port('s3')}"
         return re.sub(
             r"<Location>\s*([a-zA-Z0-9\-]+)://[^/]+/([^<]+)\s*</Location>",
             r"<Location>%s://%s/%s/\2</Location>" % (get_service_protocol(), host, bucket_name),
@@ -1214,14 +1218,14 @@ class ProxyListenerS3(PersistingProxyListener):
 
     def forward_request(self, method, path, data, headers):
         # Create list of query parameteres from the url
-        parsed = urlparse.urlparse("{}{}".format(config.get_edge_url(), path))
+        parsed = urlparse("{}{}".format(config.get_edge_url(), path))
         query_params = parse_qs(parsed.query)
         path_orig = path
         path = path.replace(
             "#", "%23"
         )  # support key names containing hashes (e.g., required by Amplify)
         # extracting bucket name from the request
-        parsed_path = urlparse.urlparse(path)
+        parsed_path = urlparse(path)
         bucket_name = extract_bucket_name(headers, parsed_path.path)
 
         if method == "PUT" and bucket_name and not re.match(BUCKET_NAME_REGEX, bucket_name):
@@ -1241,8 +1245,8 @@ class ProxyListenerS3(PersistingProxyListener):
             )
 
         # Detecting pre-sign url and checking signature
-        if any([p in query_params for p in SIGNATURE_V2_PARAMS]) or any(
-            [p in query_params for p in SIGNATURE_V4_PARAMS]
+        if any(p in query_params for p in SIGNATURE_V2_PARAMS) or any(
+            p in query_params for p in SIGNATURE_V4_PARAMS
         ):
             response = authenticate_presign_url(
                 method=method, path=path, data=data, headers=headers
@@ -1290,7 +1294,7 @@ class ProxyListenerS3(PersistingProxyListener):
         # parse query params
         query = parsed_path.query
         path = parsed_path.path
-        query_map = urlparse.parse_qs(query, keep_blank_values=True)
+        query_map = parse_qs(query, keep_blank_values=True)
 
         # remap metadata query params (not supported in moto) to request headers
         append_metadata_headers(method, query_map, headers)
@@ -1410,7 +1414,7 @@ class ProxyListenerS3(PersistingProxyListener):
                 response.status_code = 200
                 return response
 
-        parsed = urlparse.urlparse(path)
+        parsed = urlparse(path)
         bucket_name_in_host = uses_host_addressing(headers)
         should_send_notifications = all(
             [
@@ -1533,7 +1537,7 @@ class ProxyListenerS3(PersistingProxyListener):
                     exists, code, body = is_bucket_available(bucket_name)
                     if not exists:
                         return no_such_bucket(bucket_name, headers.get("x-amz-request-id"), 404)
-                query_map = urlparse.parse_qs(parsed.query, keep_blank_values=True)
+                query_map = parse_qs(parsed.query, keep_blank_values=True)
                 for param_name, header_name in ALLOWED_HEADER_OVERRIDES.items():
                     if param_name in query_map:
                         response.headers[header_name] = query_map[param_name][0]
@@ -1615,7 +1619,7 @@ def serve_static_website(headers, path, bucket_name):
             path = path.lstrip("/")
             return respond_with_key(status_code=200, key=path)
     except ClientError:
-        LOGGER.debug("No such key found. %s" % path)
+        LOGGER.debug("No such key found. %s", path)
 
     website_config = s3_client.get_bucket_website(Bucket=bucket_name)
     path_suffix = website_config.get("IndexDocument", {}).get("Suffix", "").lstrip("/")
