@@ -24,11 +24,13 @@ import uuid
 import zipfile
 from contextlib import closing
 from datetime import date, datetime, timezone, tzinfo
+from json import JSONDecodeError
 from multiprocessing.dummy import Pool
 from queue import Queue
 from typing import Any, Callable, Dict, List, Optional, Sized, Tuple, Type, Union
 from urllib.parse import parse_qs, urlparse
 
+import cachetools
 import dns.resolver
 import requests
 import six
@@ -94,6 +96,9 @@ _unprintables = (
 REGEX_UNPRINTABLE_CHARS = re.compile(
     f"[{re.escape(''.join(map(chr, itertools.chain(*_unprintables))))}]"
 )
+IP_REGEX = (
+    r"^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$"
+)
 
 # user of the currently running process
 CACHED_USER = None
@@ -185,8 +190,7 @@ class ShellCommandThread(FuncThread):
             ):
                 return self.process.returncode if self.process else None
             LOG.info(
-                "Restarting process (received exit code %s): %s"
-                % (self.process.returncode, self.cmd)
+                "Restarting process (received exit code %s): %s", self.process.returncode, self.cmd
             )
 
     def do_run_cmd(self):
@@ -247,9 +251,9 @@ class ShellCommandThread(FuncThread):
         except Exception as e:
             self.result_future.set_exception(e)
             if self.process and not self.quiet:
-                LOG.warning('Shell command error "%s": %s' % (e, self.cmd))
+                LOG.warning('Shell command error "%s": %s', e, self.cmd)
         if self.process and not self.quiet and self.process.returncode != 0:
-            LOG.warning('Shell command exit code "%s": %s' % (self.process.returncode, self.cmd))
+            LOG.warning('Shell command exit code "%s": %s', self.process.returncode, self.cmd)
 
     def is_killed(self):
         if not self.process:
@@ -267,7 +271,7 @@ class ShellCommandThread(FuncThread):
         if self.stopped:
             return
         if not self.process:
-            LOG.warning("No process found for command '%s'" % self.cmd)
+            LOG.warning("No process found for command '%s'", self.cmd)
             return
 
         parent_pid = self.process.pid
@@ -523,6 +527,64 @@ class FileMappedDocument(dict):
             json.dump(self, fd)
 
 
+class PortNotAvailableException(Exception):
+    """Exception which indicates that the ExternalServicePortsManager could not reserve a port."""
+
+    pass
+
+
+class ExternalServicePortsManager:
+    """Manages the ports used for starting external services like ElasticSearch, OpenSearch,..."""
+
+    def __init__(self):
+        # cache for locally available ports (ports are reserved for a short period of a few seconds)
+        self._PORTS_CACHE = cachetools.TTLCache(maxsize=100, ttl=6)
+        self._PORTS_LOCK = threading.RLock()
+
+    def reserve_port(self, port: int = None) -> int:
+        """
+        Reserves the given port (if it is still free). If the given port is None, it reserves a free port from the
+        configured port range for external services. If a port is given, it has to be within the configured
+        range of external services (i.e. in [config#EXTERNAL_SERVICE_PORTS_START, config#EXTERNAL_SERVICE_PORTS_END)).
+        :param port: explicit port to check or None if a random port from the configured range should be selected
+        :return: reserved, free port number (int)
+        :raises: PortNotAvailableException if the given port is outside the configured range, it is already bound or
+                    reserved, or if the given port is none and there is no free port in the configured service range.
+        """
+        ports_range = range(config.EXTERNAL_SERVICE_PORTS_START, config.EXTERNAL_SERVICE_PORTS_END)
+        if port is not None and port not in ports_range:
+            raise PortNotAvailableException(
+                f"The requested port ({port}) is not in the configured external "
+                f"service port range ({ports_range})."
+            )
+        with self._PORTS_LOCK:
+            if port is not None:
+                return self._check_port(port)
+            else:
+                for port_in_range in ports_range:
+                    try:
+                        return self._check_port(port_in_range)
+                    except PortNotAvailableException:
+                        # We ignore the fact that this single port is reserved, we just check the next one
+                        pass
+        raise PortNotAvailableException(
+            "No free network ports available to start service instance (currently reserved: %s)",
+            list(self._PORTS_CACHE.keys()),
+        )
+
+    def _check_port(self, port: int) -> int:
+        """Checks if the given port is currently not reserved and can be bound."""
+        if not self._PORTS_CACHE.get(port) and port_can_be_bound(port):
+            # reserve the port for a short period of time
+            self._PORTS_CACHE[port] = "__reserved__"
+            return port
+        else:
+            raise PortNotAvailableException(f"The given port ({port}) is already reserved.")
+
+
+external_service_ports = ExternalServicePortsManager()
+
+
 # ----------------
 # UTILITY METHODS
 # ----------------
@@ -631,7 +693,7 @@ def md5(string: Union[str, bytes]) -> str:
 def select_attributes(obj: Dict, attributes: List[str]) -> Dict:
     """Select a subset of attributes from the given dict (returns a copy)"""
     attributes = attributes if is_list_or_tuple(attributes) else [attributes]
-    return dict([(k, v) for k, v in obj.items() if k in attributes])
+    return {k: v for k, v in obj.items() if k in attributes}
 
 
 def remove_attributes(obj: Dict, attributes: List[str], recursive: bool = False) -> Dict:
@@ -907,7 +969,9 @@ def poll_condition(condition, timeout: float = None, interval: float = 0.5) -> b
     return True
 
 
-def merge_recursive(source, destination, none_values=[None], overwrite=False):
+def merge_recursive(source, destination, none_values=None, overwrite=False):
+    if none_values is None:
+        none_values = [None]
     for key, value in source.items():
         if isinstance(value, dict):
             # get node or create one
@@ -959,7 +1023,7 @@ def keys_to_lower(obj: JsonComplexType, skip_children_of: List[str] = None) -> J
     skip_children_of = ensure_list(skip_children_of or [])
 
     def fix_keys(o, path="", **kwargs):
-        if any([re.match(r"(^|.*\.)%s($|[.\[].*)" % k, path) for k in skip_children_of]):
+        if any(re.match(r"(^|.*\.)%s($|[.\[].*)" % k, path) for k in skip_children_of):
             return o
         if isinstance(o, dict):
             for k, v in dict(o).items():
@@ -971,8 +1035,11 @@ def keys_to_lower(obj: JsonComplexType, skip_children_of: List[str] = None) -> J
     return result
 
 
+_camel_to_snake_case_sub = re.compile("((?<=[a-z0-9])[A-Z]|(?!^)[A-Z](?=[a-z]))")
+
+
 def camel_to_snake_case(string: str) -> str:
-    return re.sub(r"(?<!^)(?=[A-Z])", "_", string).replace("__", "_").lower()
+    return _camel_to_snake_case_sub.sub(r"_\1", string).replace("__", "_").lower()
 
 
 def snake_to_camel_case(string: str, capitalize_first: bool = True) -> str:
@@ -1050,7 +1117,7 @@ def ensure_readable(file_path: str, default_perms: int = None):
         with open(file_path, "rb"):
             pass
     except Exception:
-        LOG.info("Updating permissions as file is currently not readable: %s" % file_path)
+        LOG.info("Updating permissions as file is currently not readable: %s", file_path)
         os.chmod(file_path, default_perms)
 
 
@@ -1143,14 +1210,7 @@ def cp_r(src: str, dst: str, rm_dest_on_conflict=False, ignore_copystat_errors=F
                 os.path.islink(_path),
             )
 
-        LOG.debug(
-            "Error copying files from %s to %s: %s"
-            % (
-                _info(src),
-                _info(dst),
-                e,
-            )
-        )
+        LOG.debug("Error copying files from %s to %s: %s", _info(src), _info(dst), e)
         raise
     finally:
         shutil.copystat = copystat_orig
@@ -1177,7 +1237,7 @@ def disk_usage(path: str) -> int:
 
 def file_exists_not_empty(path: str) -> bool:
     """Return whether the given file or directory exists and is non-empty (i.e., >0 bytes content)"""
-    return disk_usage(path) > 0
+    return path and disk_usage(path) > 0
 
 
 def format_bytes(count: float, default: str = "n/a"):
@@ -1229,8 +1289,7 @@ def download(url: str, path: str, verify_ssl: bool = True, timeout: float = None
         if not os.path.exists(os.path.dirname(path)):
             os.makedirs(os.path.dirname(path))
         LOG.debug(
-            "Starting download from %s to %s (%s bytes)"
-            % (url, path, r.headers.get("Content-Length"))
+            "Starting download from %s to %s (%s bytes)", url, path, r.headers.get("Content-Length")
         )
         with open(path, "wb") as f:
             iter_length = 0
@@ -1241,18 +1300,18 @@ def download(url: str, path: str, verify_ssl: bool = True, timeout: float = None
                 if chunk:  # filter out keep-alive new chunks
                     f.write(chunk)
                 else:
-                    LOG.debug("Empty chunk %s (total %s) from %s" % (chunk, total, url))
+                    LOG.debug("Empty chunk %s (total %s) from %s", chunk, total, url)
                 if iter_length >= iter_limit:
-                    LOG.debug("Written %s bytes (total %s) to %s" % (iter_length, total, path))
+                    LOG.debug("Written %s bytes (total %s) to %s", iter_length, total, path)
                     iter_length = 0
             f.flush()
             os.fsync(f)
         if os.path.getsize(path) == 0:
-            LOG.warning("Zero bytes downloaded from %s, retrying" % url)
+            LOG.warning("Zero bytes downloaded from %s, retrying", url)
             download(url, path, verify_ssl)
             return
         LOG.debug(
-            "Done downloading %s, response code %s, total bytes %d" % (url, r.status_code, total)
+            "Done downloading %s, response code %s, total bytes %d", url, r.status_code, total
         )
     except requests.exceptions.ReadTimeout as e:
         raise TimeoutError(f"Timeout ({timeout}) reached on download: {url} - {e}")
@@ -1282,7 +1341,7 @@ def parse_request_data(method: str, path: str, data=None, headers=None) -> Dict:
             pass  # probably binary / JSON / non-URL encoded payload - ignore
 
     # select first elements from result lists (this is assuming we are not using parameter lists!)
-    result = dict([(k, v[0]) for k, v in result.items()])
+    result = {k: v[0] for k, v in result.items()}
     return result
 
 
@@ -1328,7 +1387,7 @@ def is_linux() -> bool:
 
 
 def is_windows() -> bool:
-    return platform.system().lower() == "windows"
+    return localstack.utils.run.is_windows()
 
 
 def is_debian() -> bool:
@@ -1400,6 +1459,19 @@ def parse_json_or_yaml(markup: str) -> JsonComplexType:
                 raise
 
 
+def try_json(data: str):
+    """
+    Tries to deserialize json input to object if possible, otherwise returns original
+    :param data: string
+    :return: deserialize version of input
+    """
+    try:
+        return json.loads(to_str(data or "{}"))
+    except JSONDecodeError:
+        LOG.warning("failed serialize to json, fallback to original")
+        return data
+
+
 def json_safe(item: JsonType) -> JsonType:
     """Return a copy of the given object (e.g., dict) that is safe for JSON dumping"""
     try:
@@ -1442,8 +1514,10 @@ def assign_to_path(target, path: str, value, delimiter: str = "."):
     parent = extract_from_jsonpointer_path(target, path_to_parent, auto_create=True)
     if not isinstance(parent, dict):
         LOG.debug(
-            'Unable to find parent (type %s) for path "%s" in object: %s'
-            % (type(parent), path, target)
+            'Unable to find parent (type %s) for path "%s" in object: %s',
+            type(parent),
+            path,
+            target,
         )
         return
     path_end = int(parts[-1]) if is_number(parts[-1]) else parts[-1]
@@ -1459,9 +1533,7 @@ def extract_from_jsonpointer_path(target, path: str, delimiter: str = "/", auto_
             if path_part == "-":
                 # special case where path is like /path/to/list/- where "/-" means "append to list"
                 continue
-            LOG.warning(
-                'Attempting to extract non-int index "%s" from list: %s' % (path_part, target)
-            )
+            LOG.warning('Attempting to extract non-int index "%s" from list: %s', path_part, target)
             return None
         target_new = target[path_part] if isinstance(target, list) else target.get(path_part)
         if target_new is None:
@@ -1679,6 +1751,15 @@ def is_ip_address(addr):
         return False
 
 
+def is_ipv4_address(address: str) -> bool:
+    """
+    Checks if passed string looks like an IPv4 address
+    :param address: Possible IPv4 address
+    :return: True if string looks like IPv4 address, False otherwise
+    """
+    return bool(re.match(IP_REGEX, address))
+
+
 def is_zip_file(content):
     stream = io.BytesIO(content)
     return zipfile.is_zipfile(stream)
@@ -1695,13 +1776,13 @@ def unzip(path, target_dir, overwrite=True):
         except Exception as e:
             error_str = truncate(str(e), max_length=200)
             LOG.info(
-                'Unable to use native "unzip" command (using fallback mechanism): %s' % error_str
+                'Unable to use native "unzip" command (using fallback mechanism): %s', error_str
             )
 
     try:
         zip_ref = zipfile.ZipFile(path, "r")
     except Exception as e:
-        LOG.warning("Unable to open zip file: %s: %s" % (path, e))
+        LOG.warning("Unable to open zip file: %s: %s", path, e)
         raise e
 
     def _unzip_file_entry(zip_ref, file_entry, target_dir):
@@ -1762,7 +1843,7 @@ def generate_ssl_cert(
     from OpenSSL import crypto
 
     def all_exist(*files):
-        return all([os.path.exists(f) for f in files])
+        return all(os.path.exists(f) for f in files)
 
     def store_cert_key_files(base_filename):
         key_file_name = "%s.key" % base_filename
@@ -1788,7 +1869,7 @@ def generate_ssl_cert(
         except Exception as e:
             # fall back to temporary files if we cannot store/overwrite the files above
             LOG.info(
-                "Error storing key/cert SSL files (falling back to random tmp file names): %s" % e
+                "Error storing key/cert SSL files (falling back to random tmp file names): %s", e
             )
             target_file_tmp = new_tmp_file()
             cert_file_name, key_file_name = store_cert_key_files(target_file_tmp)
@@ -1860,8 +1941,9 @@ def generate_ssl_cert(
                     if i > 0:
                         raise
                     LOG.info(
-                        "Unable to store certificate file under %s, using tmp file instead: %s"
-                        % (target_file, e)
+                        "Unable to store certificate file under %s, using tmp file instead: %s",
+                        target_file,
+                        e,
                     )
                     # Fix for https://github.com/localstack/localstack/issues/1743
                     target_file = "%s.pem" % new_tmp_file()
@@ -1910,7 +1992,7 @@ def run_safe(_python_lambda, *args, _default=None, **kwargs):
         return _python_lambda(*args, **kwargs)
     except Exception as e:
         if print_error:
-            LOG.warning("Unable to execute function: %s" % e)
+            LOG.warning("Unable to execute function: %s", e)
         return _default
 
 
@@ -2157,6 +2239,10 @@ def get_all_subclasses(clazz: Type) -> List[Type]:
     return result
 
 
+def fully_qualified_class_name(klass: Type) -> str:
+    return f"{klass.__module__}.{klass.__name__}"
+
+
 def parallelize(func: Callable, arr: List, size: int = None):
     if not size:
         size = len(arr)
